@@ -66,19 +66,6 @@ static void htpasswd_clear(auth_t *self) {
     free(state);
 }
 
-static int get_line(FILE *file, char *buf, int len)
-{
-    if(fgets(buf, len, file)) {
-        int len = strlen(buf);
-        if(len > 0 && buf[len-1] == '\n') {
-            buf[--len] = 0;
-            if(len > 0 && buf[len-1] == '\r')
-                buf[--len] = 0;
-        }
-        return 1;
-    }
-    return 0;
-}
 
 /* md5 hash */
 static char *get_hash(const char *data, int len)
@@ -88,15 +75,13 @@ static char *get_hash(const char *data, int len)
 
     MD5Init(&context);
 
-    MD5Update(&context, data, len);
+    MD5Update(&context, (const unsigned char *)data, len);
 
     MD5Final(digest, &context);
 
     return util_bin_to_hex(digest, 16);
 }
 
-#define MAX_LINE_LEN 512
-
 
 static int compare_users (void *arg, void *a, void *b)
 {
@@ -200,6 +185,11 @@ static auth_result htpasswd_auth (auth_client *auth_user)
     if (client->username == NULL || client->password == NULL)
         return AUTH_FAILED;
 
+    if (htpasswd->filename == NULL)
+    {
+        ERROR0("No filename given in options for authenticator.");
+        return AUTH_FAILED;
+    }
     htpasswd_recheckfile (htpasswd);
 
     thread_rwlock_rlock (&htpasswd->file_rwlock);
@@ -240,19 +230,20 @@ int  auth_get_htpasswd_auth (auth_t *authenticator, config_options_t *options)
 
     while(options) {
         if(!strcmp(options->name, "filename"))
+        {
+            free (state->filename);
             state->filename = strdup(options->value);
+        }
         options = options->next;
     }
 
-    if(!state->filename) {
-        free(state);
+    if (state->filename)
+        INFO1("Configured htpasswd authentication using password file \"%s\"", 
+                state->filename);
+    else
         ERROR0("No filename given in options for authenticator.");
-        return -1;
-    }
 
     authenticator->state = state;
-    DEBUG1("Configured htpasswd authentication using password file %s", 
-            state->filename);
 
     thread_rwlock_create(&state->file_rwlock);
     htpasswd_recheckfile (state);
@@ -406,9 +397,9 @@ static auth_result htpasswd_userlist(auth_t *auth, xmlNodePtr srcnode)
     while (node)
     {
         htpasswd_user *user = (htpasswd_user *)node->key;
-        newnode = xmlNewChild (srcnode, NULL, "User", NULL);
-        xmlNewChild(newnode, NULL, "username", user->name);
-        xmlNewChild(newnode, NULL, "password", user->pass);
+        newnode = xmlNewChild (srcnode, NULL, XMLSTR("User"), NULL);
+        xmlNewChild(newnode, NULL, XMLSTR("username"), XMLSTR(user->name));
+        xmlNewChild(newnode, NULL, XMLSTR("password"), XMLSTR(user->pass));
         node = avl_get_next (node);
     }
     thread_rwlock_unlock (&state->file_rwlock);
