@@ -61,7 +61,7 @@ static int  format_mp3_create_client_data (source_t *source, client_t *client);
 static void free_mp3_client_data (client_t *client);
 static int format_mp3_write_buf_to_client(client_t *client);
 static void write_mp3_to_file (struct source_tag *source, refbuf_t *refbuf);
-static void mp3_set_tag (format_plugin_t *plugin, char *tag, char *value);
+static void mp3_set_tag (format_plugin_t *plugin, const char *tag, const char *in_value, const char *charset);
 static void format_mp3_apply_settings(client_t *client, format_plugin_t *format, mount_proxy *mount);
 
 
@@ -75,7 +75,7 @@ typedef struct {
 
 int format_mp3_get_plugin (source_t *source)
 {
-    char *metadata;
+    const char *metadata;
     format_plugin_t *plugin;
     mp3_state *state = calloc(1, sizeof(mp3_state));
     refbuf_t *meta;
@@ -124,42 +124,43 @@ int format_mp3_get_plugin (source_t *source)
 }
 
 
-static void mp3_set_tag (format_plugin_t *plugin, char *tag, char *value)
+static void mp3_set_tag (format_plugin_t *plugin, const char *tag, const char *in_value, const char *charset)
 {
     mp3_state *source_mp3 = plugin->_state;
     unsigned int len;
     const char meta[] = "StreamTitle='";
     int size = sizeof (meta) + 1;
+    char *value;
 
-    if (tag==NULL || value == NULL)
+    if (tag==NULL || in_value == NULL)
         return;
 
+    /* protect against multiple updaters */
+    thread_mutex_lock (&source_mp3->url_lock);
+
+    value = util_conv_string (in_value, charset, plugin->charset);
+    if (value == NULL)
+        value = strdup (in_value);
+
     len = strlen (value)+1;
     size += len;
-    /* protect against multiple updaters */
-    thread_mutex_lock (&source_mp3->url_lock);
+
     if (strcmp (tag, "title") == 0 || strcmp (tag, "song") == 0)
     {
-        char *p = strdup (value);
-        if (p)
-        {
-            free (source_mp3->url_title);
-            free (source_mp3->url_artist);
-            source_mp3->url_artist = NULL;
-            source_mp3->url_title = p;
-            source_mp3->update_metadata = 1;
-        }
+        free (source_mp3->url_title);
+        free (source_mp3->url_artist);
+        source_mp3->url_artist = NULL;
+        source_mp3->url_title = value;
+        source_mp3->update_metadata = 1;
     }
     else if (strcmp (tag, "artist") == 0)
     {
-        char *p = strdup (value);
-        if (p)
-        {
-            free (source_mp3->url_artist);
-            source_mp3->url_artist = p;
-            source_mp3->update_metadata = 1;
-        }
+        free (source_mp3->url_artist);
+        source_mp3->url_artist = value;
+        source_mp3->update_metadata = 1;
     }
+    else
+        free (value);
     thread_mutex_unlock (&source_mp3->url_lock);
 }
 
@@ -176,7 +177,7 @@ static void filter_shoutcast_metadata (source_t *source, char *metadata, unsigne
             metadata++;
             if (strncmp (metadata, "StreamTitle='", 13))
                 break;
-            if ((end = strstr (metadata, "\';")) == NULL)
+            if ((end = strstr (metadata+13, "\';")) == NULL)
                 break;
             len = (end - metadata) - 13;
             p = calloc (1, len+1);
@@ -184,7 +185,7 @@ static void filter_shoutcast_metadata (source_t *source, char *metadata, unsigne
             {
                 memcpy (p, metadata+13, len);
                 logging_playlist (source->mount, p, source->listeners);
-                stats_event (source->mount, "title", p);
+                stats_event_conv (source->mount, "title", p, source->format->charset);
                 yp_touch (source->mount);
                 free (p);
             }
@@ -197,10 +198,21 @@ static void format_mp3_apply_settings (client_t *client, format_plugin_t *format
 {
     mp3_state *source_mp3 = format->_state;
 
-    if (mount == NULL || mount->mp3_meta_interval < 0)
+    source_mp3->interval = -1;
+    free (format->charset);
+    format->charset = NULL;
+
+    if (mount)
     {
-        char *metadata = httpp_getvar (client->parser, "icy-metaint");
-        source_mp3->interval = -1;
+        if (mount->mp3_meta_interval > 0)
+            source_mp3->interval = mount->mp3_meta_interval;
+        if (mount->charset)
+            format->charset = strdup (mount->charset);
+    }
+    if (source_mp3->interval <= 0)
+    {
+        const char *metadata = httpp_getvar (client->parser, "icy-metaint");
+        source_mp3->interval = ICY_METADATA_INTERVAL;
         if (metadata)
         {
             int interval = atoi (metadata);
@@ -208,9 +220,12 @@ static void format_mp3_apply_settings (client_t *client, format_plugin_t *format
                 source_mp3->interval = interval;
         }
     }
-    else
-        source_mp3->interval = mount->mp3_meta_interval;
-    DEBUG1 ("mp3 interval %d", source_mp3->interval);
+
+    if (format->charset == NULL)
+        format->charset = strdup ("ISO8859-1");
+
+    DEBUG1 ("sending metadata interval %d", source_mp3->interval);
+    DEBUG1 ("charset %s", format->charset);
 }
 
 
@@ -277,7 +292,7 @@ static void mp3_set_title (source_t *source)
 static int send_mp3_metadata (client_t *client, refbuf_t *associated)
 {
     int ret = 0;
-    unsigned char *metadata;
+    char *metadata;
     int meta_len;
     mp3_client_data *client_mp3 = client->format_data;
 
@@ -411,6 +426,7 @@ static void format_mp3_free_plugin(format_plugin_t *self)
     thread_mutex_destroy (&state->url_lock);
     free (state->url_artist);
     free (state->url_title);
+    free (self->charset);
     refbuf_release (state->metadata);
     refbuf_release (state->read_data);
     free(state);
@@ -509,7 +525,7 @@ static refbuf_t *mp3_get_filter_meta (source_t *source)
 
     refbuf = source_mp3->read_data;
     source_mp3->read_data = NULL;
-    src = refbuf->data;
+    src = (unsigned char *)refbuf->data;
 
     if (source_mp3->update_metadata)
     {
@@ -622,16 +638,24 @@ static int format_mp3_create_client_data(source_t *source, client_t *client)
     unsigned remaining = 4096 - client->refbuf->len + 2;
     char *ptr = client->refbuf->data + client->refbuf->len - 2;
     int bytes;
+    const char *useragent;
 
     if (client_mp3 == NULL)
         return -1;
 
-    /* hack for flash player, it wants a length */
-    if (httpp_getvar(client->parser, "x-flash-version"))
+    /* hack for flash player, it wants a length.  It has also been reported that the useragent
+     * appears as MSIE if run in internet explorer */
+    useragent = httpp_getvar (client->parser, "user-agent");
+    if (httpp_getvar(client->parser, "x-flash-version") ||
+            (useragent && strstr(useragent, "MSIE")))
     {
-        bytes = snprintf (ptr, remaining, "Content-Length: 347122319\r\n");
+        bytes = snprintf (ptr, remaining, "Content-Length: 221183499\r\n");
         remaining -= bytes;
         ptr += bytes;
+        /* avoid browser caching, reported via forum */
+        bytes = snprintf (ptr, remaining, "Expires: Mon, 26 Jul 1997 05:00:00 GMT\r\n");
+        remaining -= bytes;
+        ptr += bytes; 
     }
 
     client->format_data = client_mp3;
